import { Location } from "./context";
import { DOMNode } from "./dom";
import { AttributeEvent, ConditionalEvent, DoctypeEvent, DOMReadyEvent, ElementReadyEvent, Event, TagCloseEvent, TagOpenEvent, WhitespaceEvent, ConfigReadyEvent } from "./event";
import { Parser } from "./parser";
import { Reporter } from "./reporter";
export interface RuleOptions {
    [key: string]: any;
}
export interface RuleDocumentation {
    description: string;
    url?: string;
}
export declare type RuleConstructor = new (options: RuleOptions) => Rule;
export declare abstract class Rule<T = any> {
    private reporter;
    private parser;
    private enabled;
    private severity;
    private event;
    /**
     * Rule name. Defaults to filename without extension but can be overwritten by
     * subclasses.
     */
    name: string;
    /**
     * Rule options.
     */
    readonly options: RuleOptions;
    constructor(options: RuleOptions);
    getSeverity(): number;
    setServerity(severity: number): void;
    setEnabled(enabled: boolean): void;
    /**
     * Returns `true` if rule is deprecated.
     *
     * Overridden by subclasses.
     */
    get deprecated(): boolean;
    /**
     * Test if rule is enabled.
     *
     * To be considered enabled the enabled flag must be true and the severity at
     * least warning.
     */
    isEnabled(): boolean;
    /**
     * Report a new error.
     *
     * Rule must be enabled both globally and on the specific node for this to
     * have any effect.
     */
    report(node: DOMNode, message: string, location?: Location, context?: T): void;
    private findLocation;
    /**
     * Listen for events.
     *
     * Adding listeners can be done even if the rule is disabled but for the
     * events to be delivered the rule must be enabled.
     *
     * @param event - Event name
     */
    on(event: "config:ready", callback: (event: ConfigReadyEvent) => void): void;
    on(event: "tag:open", callback: (event: TagOpenEvent) => void): void;
    on(event: "tag:close", callback: (event: TagCloseEvent) => void): void;
    on(event: "element:ready", callback: (event: ElementReadyEvent) => void): void;
    on(event: "dom:load", callback: (event: Event) => void): void;
    on(event: "dom:ready", callback: (event: DOMReadyEvent) => void): void;
    on(event: "doctype", callback: (event: DoctypeEvent) => void): void;
    on(event: "attr", callback: (event: AttributeEvent) => void): void;
    on(event: "whitespace", callback: (event: WhitespaceEvent) => void): void;
    on(event: "conditional", callback: (event: ConditionalEvent) => void): void;
    on(event: "*", callback: (event: Event) => void): void;
    /**
     * Called by [[Engine]] when initializing the rule.
     *
     * Do not override this, use the `setup` callback instead.
     *
     * @hidden
     */
    init(parser: Parser, reporter: Reporter, severity: number): void;
    /**
     * Rule setup callback.
     *
     * Override this to provide rule setup code.
     */
    abstract setup(): void;
    /**
     * Rule documentation callback.
     *
     * Called when requesting additional documentation for a rule. Some rules
     * provide additional context to provide context-aware suggestions.
     *
     * @param context - Error context given by a reported error.
     * @returns Rule documentation and url with additional details or `null` if no
     * additional documentation is available.
     */
    documentation(context?: T): RuleDocumentation;
}
export declare function ruleDocumentationUrl(filename: string): string;
